package cd.casic.ci.process.process.service.resource.impl;

import cd.casic.ci.process.dal.req.resource.ResourceQueryReq;
import cd.casic.ci.process.dal.req.resource.ResourceReq;
import cd.casic.ci.process.dal.resp.resource.ResourceFindResp;
import cd.casic.ci.process.enums.ResourceTypeEnum;
import cd.casic.ci.process.process.converter.ResourceConverter;
import cd.casic.ci.process.process.dal.pipeline.PipResourceCloudDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceDockerEndpointDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceK8SDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceMachineDao;
import cd.casic.ci.process.process.dal.pipeline.PipResourceTempDao;
import cd.casic.ci.process.process.dataObject.resource.PipResourceCloud;
import cd.casic.ci.process.process.dataObject.resource.PipResourceK8S;
import cd.casic.ci.process.process.dataObject.resource.PipResourceMachine;
import cd.casic.ci.process.process.dataObject.resource.PipResourceTemp;
import cd.casic.ci.process.process.dataObject.resource.PipelineDockerEndpoint;
import cd.casic.ci.process.process.service.resource.ResourceService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.security.dal.user.AdminUserDO;
import cd.casic.framework.tenant.core.service.AdminUserServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName ResourceServiceImpl
 * @Date: 2025/5/13 10:31
 * @Description:
 */
@Service
@Slf4j
public class ResourceServiceImpl extends ServiceImpl<PipResourceTempDao, PipResourceTemp> implements ResourceService {
    @Resource
    private PipResourceTempDao tempDao;

    @Resource
    private PipResourceMachineDao machineDao;

    @Resource
    private PipResourceDockerEndpointDao dockerEndpointDao;

    @Resource
    private PipResourceK8SDao k8SDao;

    @Resource
    private PipResourceCloudDao cloudDao;

    @Resource
    private AdminUserServiceImpl adminUserService;
    @Autowired
    private PipResourceDockerEndpointDao pipResourceDockerEndpointDao;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createResource(ResourceReq resourceReq) {
        if (ObjectUtils.isEmpty(resourceReq.getResourceName())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"资源类型不可为空");
        }

        PipResourceTemp pipResourceTemp = new PipResourceTemp();
        pipResourceTemp.setResourceName(resourceReq.getResourceName());
        pipResourceTemp.setResourceType(resourceReq.getResourceType());

        //如果资源类型不属于弹性云，则新增，如果属于弹性云，则走申请流程
        if (resourceReq.getResourceType().equals(ResourceTypeEnum.MACHINE.getCode())){

            if (!ObjectUtils.isEmpty(resourceReq.getResourceMachine())){
                PipResourceMachine resourceMachine = resourceReq.getResourceMachine();
                machineDao.insert(resourceMachine);

                pipResourceTemp.setResourceId(resourceMachine.getId());
            }
            tempDao.insert(pipResourceTemp);

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.K8S.getCode())) {

            if (!ObjectUtils.isEmpty(resourceReq.getPipResourceK8S())) {
                PipResourceK8S pipResourceK8S = resourceReq.getPipResourceK8S();
                k8SDao.insert(pipResourceK8S);

                pipResourceTemp.setResourceId(pipResourceK8S.getId());
            }
            tempDao.insert(pipResourceTemp);

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.DOCKER.getCode())) {

            if (!ObjectUtils.isEmpty(resourceReq.getDockerEndpoint())) {
                PipelineDockerEndpoint dockerEndpoint = resourceReq.getDockerEndpoint();
                dockerEndpointDao.insert(dockerEndpoint);

                pipResourceTemp.setResourceId(dockerEndpoint.getId());
            }
            tempDao.insert(pipResourceTemp);

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.CLOUD.getCode())){
            PipResourceCloud pipResourceCloud = resourceReq.getPipResourceCloud();
            cloudDao.insert(pipResourceCloud);

            pipResourceTemp.setResourceId(pipResourceCloud.getId());

        }

        return pipResourceTemp.getId();
    }

    @Override
    public void deleteResource(String id) {
        PipResourceTemp pipResourceTemp = tempDao.selectById(id);
        if (pipResourceTemp.getResourceType().equals(ResourceTypeEnum.MACHINE.getCode())){
            machineDao.deleteById(pipResourceTemp.getResourceId());
        }else if (pipResourceTemp.getResourceType().equals(ResourceTypeEnum.K8S.getCode())) {
            k8SDao.deleteById(pipResourceTemp.getResourceId());
        }else if (pipResourceTemp.getResourceType().equals(ResourceTypeEnum.DOCKER.getCode())) {
            dockerEndpointDao.deleteById(pipResourceTemp.getResourceId());
        } else if (pipResourceTemp.getResourceType().equals(ResourceTypeEnum.CLOUD.getCode())) {
            cloudDao.deleteById(pipResourceTemp.getResourceId());
        }
        tempDao.deleteById(id);
    }

    @Override
    public void updateResource(ResourceReq resourceReq) {
        if (ObjectUtils.isEmpty(resourceReq.getResourceType())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"资源类型不可为空");
        }

        PipResourceTemp temp = tempDao.selectById(resourceReq.getId());
        temp.setResourceName(resourceReq.getResourceName());

        //先删除旧的资源信息
        if (temp.getResourceType().equals(ResourceTypeEnum.MACHINE.getCode())){
            machineDao.deleteById(temp.getResourceId());

        }else if (temp.getResourceType().equals(ResourceTypeEnum.K8S.getCode())) {
            k8SDao.deleteById(temp.getResourceId());
            if (!ObjectUtils.isEmpty(resourceReq.getPipResourceK8S())) {
                PipResourceK8S pipResourceK8S = resourceReq.getPipResourceK8S();
                k8SDao.updateById(pipResourceK8S);
            }

        }else if (temp.getResourceType().equals(ResourceTypeEnum.DOCKER.getCode())) {
            dockerEndpointDao.deleteById(temp.getResourceId());
            if (!ObjectUtils.isEmpty(resourceReq.getDockerEndpoint())) {
                PipelineDockerEndpoint dockerEndpoint = resourceReq.getDockerEndpoint();
                dockerEndpointDao.updateById(dockerEndpoint);
            }

        }else if (temp.getResourceType().equals(ResourceTypeEnum.CLOUD.getCode())){
            cloudDao.deleteById(temp.getResourceId());
        }


        //根据新资源信息新增
        if (resourceReq.getResourceType().equals(ResourceTypeEnum.MACHINE.getCode())){

             //新增
             if (!ObjectUtils.isEmpty(resourceReq.getResourceMachine())){
                 PipResourceMachine resourceMachine = resourceReq.getResourceMachine();
                 machineDao.insert(resourceMachine);
                 temp.setResourceId(resourceMachine.getId());
             }

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.K8S.getCode())){

            //新增
            if (!ObjectUtils.isEmpty(resourceReq.getPipResourceK8S())){
                PipResourceK8S resourceK8S = resourceReq.getPipResourceK8S();
                k8SDao.insert(resourceK8S);
                temp.setResourceId(resourceK8S.getId());
            }

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.DOCKER.getCode())){

            //新增
            if (!ObjectUtils.isEmpty(resourceReq.getDockerEndpoint())){
                PipelineDockerEndpoint dockerEndpoint = resourceReq.getDockerEndpoint();
                dockerEndpointDao.insert(dockerEndpoint);
                temp.setResourceId(dockerEndpoint.getId());
            }

        }else if (resourceReq.getResourceType().equals(ResourceTypeEnum.CLOUD.getCode())){

            //新增
            if (!ObjectUtils.isEmpty(resourceReq.getPipResourceCloud())){
                PipResourceCloud resourceCloud = resourceReq.getPipResourceCloud();
                cloudDao.insert(resourceCloud);
                temp.setResourceId(resourceCloud.getId());
            }

        }
        temp.setResourceType(resourceReq.getResourceType());
        tempDao.updateById(temp);
    }

    @Override
    public ResourceFindResp findResourceById(String id) {
        ResourceFindResp resp = new ResourceFindResp();

        PipResourceTemp pipResourceTemp = tempDao.selectById(id);
        if (ObjectUtils.isEmpty(pipResourceTemp)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"资源不存在");
        }
        resp.setId(pipResourceTemp.getId());
        resp.setResourceType(pipResourceTemp.getResourceType());
        resp.setResourceId(pipResourceTemp.getResourceId());
        resp.setResourceName(pipResourceTemp.getResourceName());

        setResource(resp);
        setUserName(resp);
        return resp;
    }



    @Override
    public List<ResourceFindResp> findResourceList(ResourceQueryReq query) {
        QueryWrapper<PipResourceTemp> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceName())){
            wrapper.like("resource_name",query.getResourceName());
        }
        if (!ObjectUtils.isEmpty(query.getResourceId())){
            wrapper.eq("resource_id",query.getResourceId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceType())){
            wrapper.eq("resource_type",query.getResourceType());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        List<PipResourceTemp> pipResourceTempList = tempDao.selectList(wrapper);

        if (ObjectUtils.isEmpty(pipResourceTempList)){
            return new ArrayList<>(0);
        }

        List<ResourceFindResp> respList = ResourceConverter.INSTANCE.toRespList(pipResourceTempList);

        //对资源进行赋值
        respList.forEach(this::setResource);
        respList.forEach(this::setUserName);

        return respList;
    }

    @Override
    public PageResult<ResourceFindResp> findResourcePage(ResourceQueryReq query) {
        Page<ResourceFindResp> respPage = new Page<>();

        QueryWrapper<PipResourceTemp> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceName())){
            wrapper.like("resource_name",query.getResourceName());
        }
        if (!ObjectUtils.isEmpty(query.getResourceId())){
            wrapper.eq("resource_id",query.getResourceId());
        }
        if (!ObjectUtils.isEmpty(query.getResourceType())){
            wrapper.eq("resource_type",query.getResourceType());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        Page<PipResourceTemp> pipResourceTempPage = tempDao.selectPage(new Page<>(query.getPageNo(), query.getPageSize()), wrapper);

        if (ObjectUtils.isEmpty(pipResourceTempPage)){
            return new PageResult<>();
        }

        List<ResourceFindResp> respList = ResourceConverter.INSTANCE.toRespList(pipResourceTempPage.getRecords());

        //对资源进行赋值
        respList.forEach(this::setResource);
        respList.forEach(this::setUserName);

        respPage.setRecords(respList);
        respPage.setTotal(pipResourceTempPage.getTotal());
        respPage.setCurrent(pipResourceTempPage.getCurrent());
        respPage.setSize(pipResourceTempPage.getSize());
        PageResult<ResourceFindResp> pageResult = new PageResult<>(respList,pipResourceTempPage.getTotal());
        return pageResult;
    }

    private void setResource(ResourceFindResp resourceFindResp) {
        if (resourceFindResp.getResourceType().equals(ResourceTypeEnum.MACHINE.getCode())){
            PipResourceMachine resourceMachine = machineDao.selectById(resourceFindResp.getResourceId());
            resourceFindResp.setResourceMachine(resourceMachine);
        }else if (resourceFindResp.getResourceType().equals(ResourceTypeEnum.K8S.getCode())) {
            PipResourceK8S pipResourceK8S = k8SDao.selectById(resourceFindResp.getResourceId());
            resourceFindResp.setPipResourceK8S(pipResourceK8S);
        }else if (resourceFindResp.getResourceType().equals(ResourceTypeEnum.DOCKER.getCode())) {
            PipelineDockerEndpoint dockerEndpoint = dockerEndpointDao.selectById(resourceFindResp.getResourceId());
            resourceFindResp.setDockerEndpoint(dockerEndpoint);
        }else if (resourceFindResp.getResourceType().equals(ResourceTypeEnum.CLOUD.getCode())) {
            PipResourceCloud resourceCloud = cloudDao.selectById(resourceFindResp.getResourceId());
            resourceFindResp.setPipResourceCloud(resourceCloud);
        }
    }


    private void setUserName(ResourceFindResp resp) {
        if (!StringUtils.isEmpty(resp.getCreator())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(resp.getCreator()));
            if (!ObjectUtils.isEmpty(user)){
                resp.setCreatorName(user.getUsername());
            }
        }

        if (!StringUtils.isEmpty(resp.getUpdater())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(resp.getUpdater()));
            if (!ObjectUtils.isEmpty(user)){
                resp.setUpdaterName(user.getUsername());
            }
        }
    }
}
